pub struct PerOwnerResolverData {
pub node_id_to_def_id: NodeMap<LocalDefId>,
pub lifetime_elision_allowed: bool,
pub label_res_map: NodeMap<NodeId>,
pub lifetimes_res_map: NodeMap<LifetimeRes>,
pub id: NodeId,
pub def_id: LocalDefId,
}Fields
node_id_to_def_id: NodeMap<LocalDefId>lifetime_elision_allowed: boolWhether lifetime elision was successful.
label_res_map: NodeMap<NodeId>Resolutions for labels. Maps from NodeId of the break/continue expression to the NodeId of their corresponding blocks or loops.
lifetimes_res_map: NodeMap<LifetimeRes>Resolutions for lifetimes.
id: NodeIdThe id of the owner
def_id: LocalDefIdThe DefId of the owner, can’t be found in node_id_to_def_id.
Implementations
Sourceimpl PerOwnerResolverData
impl PerOwnerResolverData
pub fn new(id: NodeId, def_id: LocalDefId) -> PerOwnerResolverData
Sourcepub fn get_label_res(&self, id: NodeId) -> Option<NodeId>
pub fn get_label_res(&self, id: NodeId) -> Option<NodeId>
Obtains resolution for a label with the given NodeId.
Sourcepub fn get_lifetime_res(&self, id: NodeId) -> Option<LifetimeRes>
pub fn get_lifetime_res(&self, id: NodeId) -> Option<LifetimeRes>
Obtains resolution for a lifetime with the given NodeId.