pub struct ResolverAstLowering<'tcx> {
pub partial_res_map: NodeMap<PartialRes>,
pub import_res_map: NodeMap<PerNS<Option<Res<NodeId>>>>,
pub extra_lifetime_params_map: NodeMap<Vec<(Ident, NodeId, MissingLifetimeKind)>>,
pub next_node_id: NodeId,
pub owners: NodeMap<PerOwnerResolverData>,
pub trait_map: NodeMap<&'tcx [TraitCandidate<'tcx>]>,
pub lint_buffer: Steal<LintBuffer>,
pub delegation_infos: LocalDefIdMap<DelegationInfo>,
pub disambiguators: LocalDefIdMap<Steal<PerParentDisambiguatorState>>,
}Expand description
Resolutions that should only be used for lowering. This struct is meant to be consumed by lowering.
Fields
partial_res_map: NodeMap<PartialRes>Resolutions for nodes that have a single resolution.
import_res_map: NodeMap<PerNS<Option<Res<NodeId>>>>Resolutions for import nodes, which have multiple resolutions in different namespaces.
extra_lifetime_params_map: NodeMap<Vec<(Ident, NodeId, MissingLifetimeKind)>>Lifetime parameters that lowering will have to introduce.
next_node_id: NodeIdowners: NodeMap<PerOwnerResolverData>trait_map: NodeMap<&'tcx [TraitCandidate<'tcx>]>lint_buffer: Steal<LintBuffer>Lints that were emitted by the resolver and early lints.
delegation_infos: LocalDefIdMap<DelegationInfo>disambiguators: LocalDefIdMap<Steal<PerParentDisambiguatorState>>Trait Implementations